/*
Copyright (C) 2003 by Mohan Embar
http://www.thisiscool.com/

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA. 
*/

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.util.Iterator;
import java.util.Set;

public class NetTest
{
public static void main(String[] args)
{
	boolean bTestInterrupt =
		args.length>0 && args[0].equalsIgnoreCase("-interrupt");
	
	for (int i=0; i<TESTS.length; ++i)
	{
		ClientServerTest aTest = TESTS[i];
		String strInternalName = aTest.getInternalName();
		if (strInternalName.indexOf("!Sun") != -1)
			continue;
		
		ITest iClientTest = aTest.getClientTest();
		ITest iServerTest = aTest.getServerTest();
		
		try
		{
			System.out.println("===========================");
			System.out.println(aTest.getDisplayName());
			
			SafeTest aServerTest = new SafeTest(iServerTest);
			SafeTest aClientTest = new SafeTest(iClientTest);
			
			aServerTest.start();
			Thread.sleep(1000);
			aClientTest.start();
			
			if
			(
				aServerTest.isInterruptibleTest()
				||
				aClientTest.isInterruptibleTest()
			)
			{
				Thread.sleep(1500);
				aServerTest.interrupt();
				aClientTest.interrupt();
			}
			
			aServerTest.join();
			aClientTest.join();
         
			aClientTest.dispose();
			aServerTest.dispose();

			System.out.println("===========================");
		}
		catch (Throwable thw)
		{
			thw.printStackTrace();
		}
	}
}

// private //
// private //
private static ClientServerTest[] TESTS =
{
	new ClientServerTest
	(
		"Simple",
		"simple test",
		new SimpleTest("TrivialClient"),
		new SimpleTest("TrivialServer")
	),
	new ClientServerTest
	(
		"Datagram",
		"datagram socket, normal outcome, infinite timeout",
		new DatagramSocketClientTest(),
		new DatagramSocketServerTest(0, false)
	),
	new ClientServerTest
	(
		"Datagram",
		"datagram socket, normal outcome, finite timeout",
		new DatagramSocketClientTest(),
		new DatagramSocketServerTest(3000, false)
	),
	new ClientServerTest
	(
		"Datagram",
		"datagram socket, times out",
		null,
		new DatagramSocketServerTest(3000, false)
	),
	new ClientServerTest
	(
		"Datagram",
		"datagram socket, interrupt, finite timeout",
		null,
		new DatagramSocketServerTest(3000, true)
	),
	new ClientServerTest
	(
		"Socket",
		"normal socket, normal outcome, infinite timeout",
		new SocketClientTest(SocketClientTest.NO_TIMEOUT, 0, false),
		new SocketServerTest(false)
	),
	new ClientServerTest
	(
		"Socket",
		"normal socket, normal outcome, finite timeout on accept()",
		new SocketClientTest(SocketClientTest.NO_TIMEOUT, 0, false),
		new SocketServerTest
		(
		  SocketServerTest.TIMEOUT_ACCEPT,
		  3000,
		  false
		)
	),
	new ClientServerTest
	(
		"Socket",
		"normal socket, times out on accept()",
		null,
		new SocketServerTest
		(
		  SocketServerTest.TIMEOUT_ACCEPT,
		  3000,
		  false
		)
	),
	new ClientServerTest
	(
		"Socket",
		"normal socket, interrupt, finite timeout on accept()",
		null,
		new SocketServerTest
		(
		  SocketServerTest.TIMEOUT_ACCEPT,
		  3000,
		  true
		)
	),
	new ClientServerTest
	(
		"Socket",
		"normal socket, normal outcome, finite timeout on read()",
		new SocketClientTest(SocketClientTest.NO_TIMEOUT, 0, false),
		new SocketServerTest
		(
		  SocketServerTest.TIMEOUT_READ,
		  3000,
		  false
		)
	),
	new ClientServerTest
	(
		"Socket",
		"normal socket, times out on read()",
		new SocketClientTest(SocketClientTest.NEVER_WRITE, 0, false),
		new SocketServerTest
		(
		  SocketServerTest.TIMEOUT_READ,
		  3000,
		  false
		)
	),
	new ClientServerTest
	(
		"Socket",
		"normal socket, interrupt, finite timeout on read()",
		new SocketClientTest(SocketClientTest.NEVER_WRITE, 0, false),
		new SocketServerTest
		(
		  SocketServerTest.TIMEOUT_READ,
		  3000,
		  true
		)
	),
	new ClientServerTest
	(
		"Socket",
		"normal socket, finite timeout on connect()",
		new SocketClientTest
		(
		  SocketClientTest.TIMEOUT_CONNECT,
		  1000,
		  false
		),
		null
	),
	new ClientServerTest
	(
		"Socket",
		"normal socket, interrupt, finite timeout on connect()",
		new SocketClientTest
		(
		  SocketClientTest.TIMEOUT_CONNECT,
		  1000,
		  true
		),
		null
	),
	new ClientServerTest
	(
		"Channels Normal Infinite",
		"channels, normal outcome, infinite timeout",
		new ChannelsClientTest(ChannelsClientTest.NO_TIMEOUT, 0, false),
		new ChannelsServerTest(false)
	),
	new ClientServerTest
	(
		"Channels Normal Finite",
		"channels, normal outcome, finite timeout",
		new ChannelsClientTest(ChannelsClientTest.NO_TIMEOUT, 0, false),
		new ChannelsServerTest(ChannelsServerTest.TIMEOUT, 3000, false)
	),
	new ClientServerTest
	(
		"Channels NoClient Infinite Interrupt",
		"channels, no client, infinite timeout, interrupt",
		null,
		new ChannelsServerTest(true)
	),
	new ClientServerTest
	(
		"Channels NoClient Timeout NoInterrupt",
		"channels, no client, timeout, no interrupt",
		null,
		new ChannelsServerTest(ChannelsServerTest.TIMEOUT, 3000, false)
	),
	new ClientServerTest
	(
		"Channels NoClient Timeout Interrupt",
		"channels, no client, timeout, interrupt",
		null,
		new ChannelsServerTest(ChannelsServerTest.TIMEOUT, 3000, true)
	),
	new ClientServerTest
	(
		"Channels NoTerm NoTimeout Interrupt",
		"channels, no termination, interrupt, infinite timeout",
		new ChannelsClientTest(ChannelsClientTest.NO_BYE, 0, false),
		new ChannelsServerTest(true)
	),
	new ClientServerTest
	(
		"Channels NoTerm Timeout Interrupt",
		"channels, no termination, interrupt, finite timeout",
		new ChannelsClientTest(ChannelsClientTest.NO_BYE, 0, false),
		new ChannelsServerTest(ChannelsServerTest.TIMEOUT, 3000, true)
	),

	// These next three will block indefinitely under Sun's JRE,
	// but not under gcj/Linux
	new ClientServerTest
	(
		"Datagram !Sun",
		"datagram, interrupt, infinite timeout",
		null,
		new DatagramSocketServerTest(0, true)
	),
	new ClientServerTest
	(
		"Socket !Sun",
		"normal socket, interrupt, infinite timeout on accept()",
		null,
		new SocketServerTest(true)
	),
	new ClientServerTest
	(
		"Socket !Sun",
		"normal socket, interrupt, infinite timeout on read()",
		new SocketClientTest(SocketClientTest.NEVER_WRITE, 0, false),
		new SocketServerTest(true)
	),
	new ClientServerTest
	(
		"Socket !Sun",
		"normal socket, interrupt, infinite timeout on connect()",
		new SocketClientTest
		(
		  SocketClientTest.TIMEOUT_CONNECT,
		  0,
		  true
		),
		null
	),
};

//////////////////////////////////////////////////////
// nested interface ITest

private static interface ITest extends Runnable
{
public String getName();
public boolean isInterruptibleTest();
public void dispose(Thread thrd);
}

// end nested interface ITest
//////////////////////////////////////////////////////

//////////////////////////////////////////////////////
// nested class ClientServerTest

private static class ClientServerTest
{
final String getInternalName()
{
	return m_strInternalName;
}

final String getDisplayName()
{
	return m_strDisplayName;
}

final ITest getClientTest()
{
	return m_iClientTest;
}

final ITest getServerTest()
{
	return m_iServerTest;
}

ClientServerTest
(
	String strInternalName,
	String strDisplayName,
	ITest iClientTest,
	ITest iServerTest
)
{
	m_strInternalName = strInternalName;
	m_strDisplayName = strDisplayName;
	m_iClientTest = iClientTest;
	m_iServerTest = iServerTest;
}

// private //
private String m_strInternalName;
private String m_strDisplayName;
private ITest m_iClientTest;
private ITest m_iServerTest;
}

// end nested class ClientServerTest
//////////////////////////////////////////////////////

//////////////////////////////////////////////////////
// nested class TestBase

private static abstract class TestBase implements ITest
{
// ITest overrides
public final String getName()
{
	return m_strName;
}

public final boolean isInterruptibleTest()
{
	return m_bInterruptible;
}

public final void dispose(Thread thrd)
{
	try
	{
		if (thrd.isInterrupted())
			out().println("--> Interrupted!");
		out().println("--> End "+m_strName);
		m_strm.close();
		m_strmBA.writeTo(System.out);
	}
	catch (IOException e)
	{
		e.printStackTrace();
	}
}

// Runnable overrides
public abstract void run();

// protected //
protected TestBase(String strName, boolean bInterruptible)
{
	m_strName = strName;
	m_bInterruptible = bInterruptible;
	
	// Set up my writers
	m_strmBA = new ByteArrayOutputStream();
	m_strm = new PrintStream(new BufferedOutputStream(m_strmBA));
}

protected final PrintStream out()
{
	return m_strm;
//	return System.out;
}

protected final void dump(Throwable thw)
{
	out().println
	(
		"EXCEPTION: "+thw.getClass().getName()+": "+thw.getMessage()
	);
}

// private //
private String m_strName;
private boolean m_bInterruptible;
private ByteArrayOutputStream m_strmBA;
private PrintStream m_strm;
}

// end nested class TestBase
//////////////////////////////////////////////////////

//////////////////////////////////////////////////////
// nested class SafeTest

private static class SafeTest
{
// package //
SafeTest(ITest iTest)
{
	m_iTest = iTest;
	m_thrd = null;
}

final void start()
{
	if (m_iTest == null)
		return;
		
	m_thrd = new Thread(m_iTest);
	m_thrd.start();
}

final void join()
{
	if (m_thrd == null)
		return;
		
	try
	{
		m_thrd.join();
	}
	catch (InterruptedException e)
	{}
}

final void interrupt()
{
	if (m_thrd!=null && m_iTest.isInterruptibleTest())
		m_thrd.interrupt();
}

final void dispose()
{
	if (m_thrd == null)
		return;
		
	m_iTest.dispose(m_thrd);
}

final boolean isInterruptibleTest()
{
	return m_iTest!=null && m_iTest.isInterruptibleTest();
}

// private //
private Thread m_thrd;
private ITest m_iTest;
}

// end nested class SafeTest
//////////////////////////////////////////////////////

//////////////////////////////////////////////////////
// nested class SimpleTest

private static class SimpleTest extends TestBase
{
// Runnable overrides
public final void run()
{
	out().println("Sleeping for 1s");
	try
	{
		Thread.sleep(1000);
		out().println("That was WONDERFUL!");
	}
	catch (InterruptedException e)
	{}
}

// package //
SimpleTest(String strName)
{
	super(strName, false);
}
}

// end nested class SimpleTest
//////////////////////////////////////////////////////

//////////////////////////////////////////////////////
// nested class DatagramSocketClientTest

private static class DatagramSocketClientTest extends TestBase
{
// Runnable overrides
public final void run()
{
	String[] STRINGS =
		new String[] {"123", "456", "789", "bye"};
	
	try
	{
		InetAddress address = InetAddress.getByName("localhost");
		
		for (int i=0; i<STRINGS.length; ++i)
		{
			try
			{
				DatagramSocket skt = new DatagramSocket();
				
				// Send request
				String strRequest = STRINGS[i];
				byte[] buf = strRequest.getBytes();
				DatagramPacket aPacket =
					new DatagramPacket(buf, buf.length, address, 4445);
				out().println("Sending: "+strRequest);
				skt.send(aPacket);

				// Retrieve and print response
				aPacket = new DatagramPacket(buf, buf.length);
				skt.receive(aPacket);
				
				String strReply = new String(aPacket.getData());
				out().println("...got "+strReply);
				
				skt.close();
			}
			catch (Exception e)
			{
				dump(e);
			}
		}
	}
	catch (Exception e)
	{
		e.printStackTrace(out());
	}
}

// package //
DatagramSocketClientTest()
{
	super("Datagram Client", false);
}
}

// end nested class DatagramSocketClientTest
//////////////////////////////////////////////////////

//////////////////////////////////////////////////////
// nested class DatagramSocketServerTest

private static class DatagramSocketServerTest extends TestBase
{
// Runnable overrides
public final void run()
{
	try
	{
		DatagramSocket skt = new DatagramSocket(4445);
		byte[] buf = new byte[256];
	
		for (;;)
		{
			try
			{
			// Receive request
			DatagramPacket aPacket =
			   new DatagramPacket(buf, buf.length);

			if (m_nTimeoutMSecs > 0)
			{
			   skt.setSoTimeout(m_nTimeoutMSecs);
			}
			skt.receive(aPacket);
	
				// Determine and send response
				String strRequest =
					new String(aPacket.getData(), 0, aPacket.getLength());
				out().println("Got "+strRequest);
				String strReply =
					new StringBuffer(strRequest).reverse().toString();
				buf = strReply.getBytes();
	
			InetAddress address = aPacket.getAddress();
			int port = aPacket.getPort();
			aPacket =
			   new DatagramPacket(buf, buf.length, address, port);
			skt.send(aPacket);

			out().println("  ...sent "+strReply);

			if (strRequest.equals("bye"))
				break;
			}
			catch (Exception e)
			{
				dump(e);
				break;
			}
		}
		
		skt.close();
	}
	catch (Exception e)
	{
		e.printStackTrace(out());
	}
}

// package //
DatagramSocketServerTest(int nTimeoutMSecs, boolean bInterruptible)
{
	super("Datagram Server", bInterruptible);
	m_nTimeoutMSecs = nTimeoutMSecs;
}

// private //
private int m_nTimeoutMSecs;
}

// end nested class DatagramSocketServerTest
//////////////////////////////////////////////////////

//////////////////////////////////////////////////////
// nested class SocketClientTest

private static class SocketClientTest extends TestBase
{
// Runnable overrides
public final void run()
{
	String[] STRINGS =
		new String[] {"ABC", "DEF", "GHI", "bye"};
	
	try
	{
		for (int i=0; i<STRINGS.length; ++i)
		{
			try
			{
				Socket skt = null;
				if (m_nMode == TIMEOUT_CONNECT)
				{
					skt = new Socket();
					skt.bind(null);
					skt.connect
					(
						new InetSocketAddress
						(
							InetAddress.getByName("1.1.1.1"),
							4444
						),
						m_nTimeoutMSecs
					);
				}
				else
				{
					skt = new Socket("localhost", 4445);
				}
				
				PrintStream strmOut =
					new PrintStream
					(
						new BufferedOutputStream
						(
							 skt.getOutputStream()
							)
					);
				BufferedReader rdrIn =
					new BufferedReader
					(
							new InputStreamReader
							(
							skt.getInputStream()
						 )
					);
				
				// Send request
				if (m_nMode == NEVER_WRITE)
				{
					Thread.sleep(4000);
				}
				else
				{
					String strRequest = STRINGS[i];
					out().println("Sending: "+strRequest);
					strmOut.println(strRequest);
					strmOut.flush();

					// Retrieve and print response
					String strReply = rdrIn.readLine();
					out().println("...got "+strReply);
				}
				
				skt.close();
				
				if (m_nMode == NEVER_WRITE)
					break;
			}
			catch (Exception e)
			{
				dump(e);
				if (m_nMode == TIMEOUT_CONNECT)
					break;
			}
		}
	}
	catch (Exception e)
	{
		e.printStackTrace(out());
	}
}

// package //
final static int NO_TIMEOUT = 0;
final static int TIMEOUT_CONNECT = 1;
final static int NEVER_WRITE = 2;

SocketClientTest(int nMode, int nTimeoutMSecs, boolean bInterruptible)
{
	super("Socket Client", bInterruptible);
	m_nMode = nMode;
	m_nTimeoutMSecs = nTimeoutMSecs;
}

// private //
private int m_nMode;
private int m_nTimeoutMSecs;
}

// end nested class SocketClientTest
//////////////////////////////////////////////////////

//////////////////////////////////////////////////////
// nested class SocketServerTest

private static class SocketServerTest extends TestBase
{
// Runnable overrides
public final void run()
{
	try
	{
		ServerSocket skt = new ServerSocket(4445);
	    
		try
		{
			for (;;)
			{
				if (m_nWhen == TIMEOUT_ACCEPT)
					skt.setSoTimeout(m_nTimeoutMSecs);

				Socket sktClient = skt.accept();
				if (m_nWhen == TIMEOUT_READ)
					sktClient.setSoTimeout(m_nTimeoutMSecs);

				PrintStream strmOut =
					new PrintStream
					(
						new BufferedOutputStream
						(
							 sktClient.getOutputStream()
							)
					);
				BufferedReader rdrIn =
					new BufferedReader
					(
							new InputStreamReader
							(
							sktClient.getInputStream()
						 )
					);
					
				// Receive request
				String strRequest = rdrIn.readLine();
				out().println("Got "+strRequest);

			// Determine and send response
			String strReply =
				new StringBuffer(strRequest).reverse().toString();
			strmOut.println(strReply);
				out().println("  ...sent "+strReply);

			strmOut.close();
			rdrIn.close();
			sktClient.close();

				if (strRequest.equals("bye"))
					 break;
			}
		}
		finally
		{
			skt.close();
		}
	}
	catch (Exception e)
	{
		dump(e);
	}
}

// package //
final static int NO_TIMEOUT = 0;
final static int TIMEOUT_ACCEPT = 1;
final static int TIMEOUT_READ = 2;

SocketServerTest(int nWhen, int nTimeoutMSecs, boolean bInterruptible)
{
	super("Socket Server", bInterruptible);
	m_nTimeoutMSecs = nTimeoutMSecs;
	m_nWhen = nWhen;
}

SocketServerTest(boolean bInterruptible)
{
	this(NO_TIMEOUT, 0, bInterruptible);
}

// private //
private int m_nTimeoutMSecs;
private int m_nWhen;
}

// end nested class SocketServerTest
//////////////////////////////////////////////////////

//////////////////////////////////////////////////////
// nested class ChannelsClientTest

private static class ChannelsClientTest extends TestBase
{
// Runnable overrides
public final void run()
{
	String[] STRINGS =
		new String[] {"ABC", "DEF", "GHI"};
	
	try
	{
		// Open up nine outbound connections, three on
		// each of ports 4445 to 4447.
		Socket[] arskt = new Socket[9];
		int nCurIndex = 0;
		for (int nPort=4445; nPort<=4447; ++nPort)
		{
			for (int i=0; i<3; ++i)
			{
				arskt[nCurIndex++] = new Socket("localhost", nPort);
			}
		}
		
		boolean bByeSent = false;
		for (int nPass=0; nPass<2; ++nPass)
		{
			for (int nSocket=0; nSocket<9; ++nSocket)
			{
				Socket skt = arskt[nSocket];
			
				try
				{
					PrintStream strmOut =
						new PrintStream(
							new BufferedOutputStream(
								skt.getOutputStream()));
					BufferedReader rdrIn =
						new BufferedReader(
							new InputStreamReader(skt.getInputStream()));

					for (int i = 0; i < STRINGS.length; ++i)
					{
						// Send request
						if (m_nMode == NEVER_WRITE)
						{
							Thread.sleep(4000);
						}
						else
						{
							String strRequest = null;
							if (nPass == 0)
								strRequest = STRINGS[i];
							else
							{
								strRequest = m_nMode==NO_BYE ? "seeya" : "bye";
							}
								
							if (!bByeSent)
							{
								out().println(
									"["
										+ ((InetSocketAddress) skt
											.getRemoteSocketAddress())
											.getPort()
										+ "] Sending: "
										+ strRequest);
								strmOut.print(strRequest);
								strmOut.flush();
	
								// Retrieve and print response
								String strReply = rdrIn.readLine();
								strReply =
									strReply.substring(0, strReply.length() - 1);
								// hack off the trailing '\n'
								out().println("...got " + strReply);
							}
							
							if (nPass == 1)
							{
								bByeSent = true;
								skt.close();
								break;
							}
						}

						if (m_nMode == NEVER_WRITE)
							break;
					}
				}
				catch (Exception e)
				{
					dump(e);
					break;
				}
			}
		}
	}
	catch (Exception e)
	{
		e.printStackTrace(out());
	}
}

// package //
final static int NO_TIMEOUT = 0;
final static int NEVER_WRITE = 1;
final static int NO_BYE = 2;

ChannelsClientTest(int nMode, int nTimeoutMSecs, boolean bInterruptible)
{
	super("Channels Client", bInterruptible);
	m_nMode = nMode;
	m_nTimeoutMSecs = nTimeoutMSecs;
}

// private //
private int m_nMode;
private int m_nTimeoutMSecs;
}

// end nested class ChannelsClientTest
//////////////////////////////////////////////////////

//////////////////////////////////////////////////////
// nested class ChannelsServerTest

private static class ChannelsServerTest extends TestBase
{
// Runnable overrides
public final void run()
{
	try
	{
		Selector sel = Selector.open();
		ByteBuffer buf = ByteBuffer.allocate(1024);

		// Add three listeners on three contiguous ports.
		ServerSocketChannel[] arssc = new ServerSocketChannel[3];
		int nCurIndex = 0; 
		for (int nPort = 4445; nPort <= 4447; ++nPort)
		{
			ServerSocketChannel ssc = ServerSocketChannel.open();
			arssc[nCurIndex++] = ssc;
			ssc.configureBlocking(false);
			ssc.socket().bind(new InetSocketAddress(nPort));
			ssc.register(sel, SelectionKey.OP_ACCEPT);
			// register interest in ACCEPT
		}

		boolean bClientShutdown = false;
    outer : while (sel.select(m_nTimeoutMSecs) > 0)
		{
			Set setReadKeys = sel.selectedKeys();
			Iterator itrReadyKeys = setReadKeys.iterator();

			while (itrReadyKeys.hasNext())
			{
				SelectionKey keysel = (SelectionKey) itrReadyKeys.next();
				itrReadyKeys.remove();

				if (keysel.isAcceptable())
				{
					ServerSocketChannel keyChannel =
						(ServerSocketChannel) keysel.channel();

					SocketChannel schn = keyChannel.accept();
					schn.configureBlocking(false);

					// register it with the selector
					schn.register(sel, SelectionKey.OP_READ);
				}
				else if (keysel.isReadable())
				{
					SocketChannel schn =
						(SocketChannel) keysel.channel();
						
					buf.clear();
					// prepare for writing into buffer

					int nBytesRead = 0;
					if ((nBytesRead = schn.read(buf)) > 0)
					{
						buf.flip();
							// prepare for reading from buffer

						byte[] arby = new byte[nBytesRead];
						buf.get(arby);

						String str = new String(arby);
						out().println("Got "+str);
						str = new StringBuffer(str).reverse().toString();
						out().println("...sending "+str);

						// Reverse this.
						arby = str.getBytes();

						buf.clear();
						buf.put(arby);
						buf.putChar('\n');
						buf.flip();

						schn.write(buf);
						
						if (str.equalsIgnoreCase("eyb"))
						{
							bClientShutdown = true;
							break outer;
						}
					}
					else
					{
						schn.close();
					}
				}
			}
		}

		if (bClientShutdown)
			out().println("...shut down by client");
		else
			out().println("...interrupted or timed out (interrupted=" +
				Thread.currentThread().isInterrupted()+")");
		
		sel.close();
		for (int i=0; i<arssc.length; ++i)
			arssc[i].close();
	}
	catch (Exception e)
	{
		dump(e);
	}
}

// package //
final static int NO_TIMEOUT = 0;
final static int TIMEOUT = 1;

ChannelsServerTest(int nWhen, int nTimeoutMSecs, boolean bInterruptible)
{
	super("Channels Server", bInterruptible);
	m_nTimeoutMSecs = nTimeoutMSecs;
	m_nWhen = nWhen;
}

ChannelsServerTest(boolean bInterruptible)
{
	this(NO_TIMEOUT, 0, bInterruptible);
}

// private //
private int m_nTimeoutMSecs = 0;
private int m_nWhen;
}

// end nested class ChannelsServerTest
//////////////////////////////////////////////////////

}
